/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009-2011 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SKELETONS_CREME_LU_LUSWEEP_H_INCLUDED
#define SKELETONS_CREME_LU_LUSWEEP_H_INCLUDED

#include <sstmac/sstmacro.h>
#include <sstmac/software/libraries/creme/creme.h>
#include <sstmac/software/libraries/creme/circuit/creme_mpi_circuit.h>

namespace lusweep
{
  class lumain : public sstmac::sw::creme_app
  {

  protected:

    /** Protected constructor, public uses lumain::construct(). */
    lumain(softwareid id) : sstmac::sw::creme_app(id)
    {
    }

    int s_nx, s_ny, s_nz, s_itr;
    double s_wg_jacu, s_wg_jacld, s_wg_buts, s_wg_blts;
    double s_wg_rhs1, s_wg_rhs2, s_wg_rhs3, s_wg_rhs4;

    sstmac::sw::CREME_MPI_Circuit::ptr toN_;
    sstmac::sw::CREME_MPI_Circuit::ptr toS_;
    sstmac::sw::CREME_MPI_Circuit::ptr toE_;
    sstmac::sw::CREME_MPI_Circuit::ptr toW_;
    sstmac::sw::CREME_MPI_Circuit::ptr fromN_;
    sstmac::sw::CREME_MPI_Circuit::ptr fromS_;
    sstmac::sw::CREME_MPI_Circuit::ptr fromE_;
    sstmac::sw::CREME_MPI_Circuit::ptr fromW_;

  public:
    typedef boost::intrusive_ptr<lumain> ptr;
    static ptr
    construct(softwareid id)
    {
      ptr obj(new lumain(id));
      return obj;
    }

    virtual
    ~lumain() throw ()
    {
    }

    app::ptr
    clone(softwareid id)
    {
      app::ptr cl = lumain::construct(id);
      return cl;
    }

    void
    skeleton_main();

    virtual void
    consume_params(const sim_parameters::ptr &params);

    virtual std::string
    to_string() const
    {
      return "lusweep-creme";
    }

    /** Overrides thread::run virtual function.
     * The task thread context is started by a call into this function. */
    void
    sweep(int pex, int pey, int size, int rank, int nx, int ny, int nz);
    void
    lower(int tilex, int tiley, int nz);
    void
    upper(int tilex, int tiley, int nz);
    void
    rhs(int tilex, int tiley, int nz);
    void
    exchange(sstmac::sw::CREME_MPI_Circuit::ptr fromcirc,
        sstmac::sw::CREME_MPI_Circuit::ptr tocirc, int recvlen, int sendlen);

    int
    get_east(int rank, int pex, int pey);
    int
    get_west(int rank, int pex, int pey);
    int
    get_south(int rank, int pex, int pey);
    int
    get_north(int rank, int pex, int pey);

  };

} // end of namespace lusweep

#endif
