/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_BACKENDS_NATIVE_DRIVER_H_INCLUDED
#define SSTMAC_BACKENDS_NATIVE_DRIVER_H_INCLUDED

#include <sstmac/common/driver_util.h>
#include <sstmac/common/config.h>

#include <sstmac/backends/native/eventlist.h>
#include <sstmac/backends/native/eventmap.h>
#include <sstmac/backends/native/manager.h>
#include <sstmac/backends/native/managerdegraded.h>

//node models
#include <sstmac/hardware/node/simplenode.h>

//nic models
#include <sstmac/hardware/nic/simplenic.h>

//event managers
#include <sstmac/backends/native/eventmap.h>

#include <sstmac/software/services/launch/allocation/allocationstrategy.h>
#include <sstmac/common/factories/network_factory.h>
#include <sstream>

namespace sstmac { namespace native
{

  using namespace std;

  eventmanager::ptr
  run(const sim_parameters::ptr &params, timestamp until, bool print)
  {

    bool errorflag = false;

    //set the global parameters object
    sstmac_env::params = params;



    interconnect::ptr fab = SSTNetworkFactory::get_network(params->get_param("network_name"), params);

    /** Set up static allocation info */
   // std::string nodefile = params->get_optional_param("launch_dumpi_mapname","");
   // sstmac::sw::allocationstrategy::set_nodemap_file(nodefile);
  //  sstmac::sw::allocationstrategy::set_topology(fab->get_topology());

    /** I would like to avoid doing this, but there are
        so many calls to parameter node_num it's not worth
        going through the code to fix them all. Just
        take the interconnect object and override
        the parameter. JJW 01/09/12
    */
    params->add_param_override("node_num", sstprintf("%d", fab->slots()));

    debug dbg_("<debug> sstmac | sim | simulation");

    manager::ptr mgr;

    try{
        std::string mgr_type = params->get_optional_param("event_manager", "eventmap");
        eventmanager::ptr ev_mgr;
        if (mgr_type == "eventmap"){
           ev_mgr = eventmap::construct();
        }

        else if (mgr_type == "eventlist"){
            long ten_million = 10000000;
            long one_million = 1000000;
            /** for ps ticks, this covers a ms */
            long nticks_window = params->get_optional_long_param("eventlist_nticks_window", one_million);
            /** for ps ticks, this covers a us */
            long nwindows = params->get_optional_long_param("eventlist_nwindows", one_million);
            ev_mgr = eventlist::construct(nticks_window, nwindows);
        }
        else
        {
            std::string errormsg = "driver::run: invalid event manager " + mgr_type;
            throw inputerror(errormsg);
        }

        if(!params->has_param("manager") || params->get_param("manager") == "simple"){
            mgr = manager::construct(ev_mgr);
        }else if(params->get_param("manager") == "degraded"){
            mgr = manager_degraded::construct(ev_mgr);
        }

        mgr->set_interconnect(fab);

        mgr->add_nodes(params);

        SSTMAC_DEBUG << "-- nodes added -- \n";


        timestamp runtime = mgr->run(until);



        if (print)
          {
            dbg_ << "final time: ";
            dbg_ << runtime.sec() << "\n";
            //dbg_ << "SST/macro ran " << mgr->get_eventmanager()->nevents() << " events\n";
          }

        sstmac_env::params = 0;
        return mgr->get_eventmanager();

    } catch (const std::exception& e) {
        if (mgr)
            mgr->done();
        throw;
    }
  }

} }

#endif
