/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/backends/native/sstevent.h>


namespace sstmac
{
    namespace native
    {
    INIT_FAST_MALLOC(sst_event);

    sst_event::sst_event(const timestamp& t, const sst_message::ptr &msg,
        const eventhandler::ptr &hand) :
      to_deliver_(msg), handler_(hand), time_(t), next(0), canceled_(false)
    {

    }

    sst_event::~sst_event()
    {

    }

    void
    sst_event::execute()
    {
      if (!canceled_)
        {
          if (to_deliver_)
            to_deliver_->set_scheduled(false);
          handler_->handle(to_deliver_);
        }
    }

    sst_event::ptr
    sst_event::construct(const timestamp& t, const sst_message::ptr &msg,
        const eventhandler::ptr &hand)
    {
      sst_event::ptr ret(new sst_event(t, msg, hand));
      return ret;
    }

    std::string
    sst_event::to_string() const
    {
      std::stringstream ss;
      if (canceled_)
        {
          ss << "canceled";
        }
      else
        {
          ss << "sst_event(handler=" << (handler_ ? handler_->to_string()
              : "null") << ",  msg = "
              << (to_deliver_ ? to_deliver_->to_string() : "null") << ")";
        }
      return ss.str();
    }

    const timestamp&
    sst_event::time() const
    {
      return time_;
    }

  }
} // end of namespace sstmac
