/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_BACKENDS_NATIVE_SSTEVENT_H_INCLUDED
#define SSTMAC_BACKENDS_NATIVE_SSTEVENT_H_INCLUDED

#include <sstmac/common/messages/sst_message.h>
#include <sstmac/common/eventhandler.h>
#include <sstmac/common/malloc.h>

namespace sstmac
{
  namespace native
  {

    class sst_event : public ptr_type, public FastMalloc<sst_event>
    {

      sst_event(const timestamp& t, const sst_message::ptr &msg,
          const eventhandler::ptr &hand);

    public:
      typedef boost::intrusive_ptr<sst_event> ptr;

      ~sst_event();

      void
      execute();

      static sst_event::ptr
      construct(const timestamp& t, const sst_message::ptr &msg,
          const eventhandler::ptr &hand);

      virtual std::string
      to_string() const;

      const timestamp&
      time() const;

      sst_event::ptr next;

      eventhandler::ptr
      get_handler()
      {
        return handler_;
      }

      sst_message::ptr
      get_message()
      {
        return to_deliver_;
      }

      void
      cancel()
      {
        if (to_deliver_)
          {
            if (to_deliver_->cancellable())
              {
                canceled_ = true;
              }
          }

      }

    protected:
      sst_message::ptr to_deliver_;
      eventhandler::ptr handler_;
      timestamp time_;

      bool canceled_;

    };

  }
} // end of namespace sstmac

#endif
