/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_BACKENDS_OMNETPP_NETWORKMODELS_RACKSWITCH_H_INCLUDED
#define SSTMAC_BACKENDS_OMNETPP_NETWORKMODELS_RACKSWITCH_H_INCLUDED

#include <omnetpp.h>

#include <sstmac/common/debug.h>
#include <sstmac/backends/omnetpp/omnetpp_params.h>
#include <string>
#include <sstmac/hardware/network/topology/topology.h>
#include <sstmac/hardware/network/switches/networkswitch.h>

#include <node/processor/eventhandlerpod.h>
#include <node/processor/payloadpod.h>
#include <node_m.h>

namespace sstmac
{
  namespace omnetpp
  {

    class rackswitch : public cSimpleModule
    {
    public:
      rackswitch()
      {
        if (!dbg_.is_real())
          {
            dbg_ = debug("<debug> switch | network");
            logger::register_logger(&dbg_);
          }
      }
      ;

      virtual
      ~rackswitch();

    protected:

      virtual void
      initialize();

      virtual void
      handleMessage(cMessage* msg);

      virtual void
      finish();

      static sstmac::hw::topology::ptr topology_;

      std::map<cGate*, int> gates_;

      sstmac::hw::networkswitch::ptr switch_;
      sstmac::sim_parameters::ptr params_;

      timestamp
      now()
      {
        timestamp t(simTime().dbl());
        return t;
      }

      int nodesPerRack;
      int numRackConnections;

      std::list<ptr_type::ptr> finishes_;

      static sstmac::debug dbg_;

      /*static std::map<cChannel*, rackswitch*> channel_map1_;
       static std::map<cChannel*, rackswitch*> channel_map2_;

       static std::map<cChannel*, int> channel_vals1_;
       static std::map<cChannel*, int> channel_vals2_;

       void
       register_channel(cChannel* ch, rackswitch* rs);
       void
       update_channel_display(cChannel* ch, rackswitch* rs, double frac);
       */
      bool displayConnections;

      class fakeeventmanager : public sstmac::eventmanager
      {
        rackswitch *parent_;

        fakeeventmanager(rackswitch *p) :
          parent_(p)
        {

        }

      public:

        static boost::intrusive_ptr<fakeeventmanager>
        construct(rackswitch *p)
        {
          boost::intrusive_ptr<fakeeventmanager> ret(new fakeeventmanager(p));
          return ret;
        }

        virtual void
        clear(const sstmac::timestamp &zero_time = sstmac::timestamp(0))
        {

        }

        virtual void
        register_finish(ptr_type::ptr p)
        {
          parent_->finishes_.push_back(p);
        }

        virtual void
        cancel_all_messages(const eventhandler::ptr &handler)
        {

        }

        virtual void
        run()
        {

        }

        virtual void
        step()
        {

        }

        virtual const sstmac::timestamp&
        now()
        {
          now_ = parent_->now();
          return now_;
        }

        /// Set off the given eventhandler at the given time.
        virtual void
        schedule(const sstmac::timestamp &start_time,
            const sstmac::eventhandler::ptr &handler,
            const sstmac::sst_message::ptr &msg)
        {
          payloadpod ppod(msg);
          eventhandlerpod epod(handler);

          EventWrapperMsg* emsg = new EventWrapperMsg();
          emsg->setMsgcontent(ppod);
          emsg->setEvhandler(epod);

          double d = start_time.sec();
          simtime_t time(d);

          simtime_t simnow(parent_->now().sec());
          //have to check for a conversion error
          if (time < simnow)
            parent_->scheduleAt(simnow, emsg);
          else
            parent_->scheduleAt(time, emsg);
        }
      };

      boost::intrusive_ptr<fakeeventmanager> fem_;
    };

  }
}
#endif
