/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009-2010 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_DRIVER_UTIL_H
#define SSTMAC_DRIVER_UTIL_H

#include <sstmac/common/debug.h>
#include <sstmac/common/errors.h>

//skeletons
//#include <sstmac/software/skeletons/skeletonfactory.h>
#include <sstmac/software/services/launch/indexing/indexstrategy.h>
#include <sstmac/hardware/nic/networkinterface.h>
#include <sstmac/software/services/launch/allocation/allocationstrategy.h>
#include <sstmac/hardware/network/congestion/interconnect.h>
#include <sstmac/hardware/network/topology/topology.h>

#include <sstmac/common/sim_parameters.h>

#include <sstmac/common/sstmac-config.h>
#include <sstmac/common/timestamp.h>
#include <sstmac/common/units.h>
#include <ostream>
#include <boost/foreach.hpp>

#include <sstmac/common/vis/vis.h>

namespace sstmac
{

  namespace sw {
    class launcher;
    class launchinfo;
  }
  //using namespace sstmac::hw;
 // using namespace sstmac;
  //using namespace sstmac::native;
  //using namespace sstmac::sw;


  class command_parameters : public sstmac::sim_parameters
  {
  protected:
    command_parameters();

  public:
    typedef boost::intrusive_ptr<command_parameters> ptr;

    static ptr
    construct();

    virtual std::string
    get_param(std::string key) const;

    virtual bool
    has_param(std::string key) const;

    virtual void
    add_param(std::string key, std::string val);

    virtual void
    add_param_override(std::string key, std::string val);

    virtual void
    combine_into(sim_parameters::ptr sp);

    virtual std::string
    to_string() const;

    virtual
    ~command_parameters();

  protected:
    typedef boost::unordered_map<std::string, std::string> maptype;
    maptype params_;
  };

  class file_parameters : public sstmac::sim_parameters
  {
  protected:
    file_parameters(std::string fname);

    void
    parse_file(std::string fname);
  public:
    typedef boost::intrusive_ptr<file_parameters> ptr;

    static ptr
    construct(std::string filename);

    void
    append_params(std::string filename);

    virtual std::string
    get_param(std::string key) const;

    virtual bool
    has_param(std::string key) const;

    virtual void
    add_param(std::string key, std::string val);

    virtual void
    add_param_override(std::string key, std::string val);

    virtual void
    combine_into(sim_parameters::ptr sp);

    virtual std::string
    to_string() const;

    virtual
    ~file_parameters();

  protected:
    typedef boost::unordered_map<std::string, std::string> maptype;
    maptype params_;
  };



  sstmac::vis::vis_engine::ptr
  construct_vis_engine(sstmac::sim_parameters::ptr params);

  double
  get_positive_float(const char *value, bool &errorflag);

  int
  get_int(const char *value, bool &errorflag);

  int
  get_positive_int(const char *value, bool &errorflag);

  std::vector<int>
  get_intvec(const char *value, bool &errorflag);

}
#endif
