/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/common/factories/allocator_factory.h>

namespace sstmac
{
  using namespace sstmac::sw;

  std::map<std::string, SSTDesc_base*>* SSTAllocatorFactory::allocatormap_ = 0;

  void
  SSTAllocatorFactory::register_allocator(std::string name, SSTDesc_base* descr)
  {
    if (allocatormap_ == 0)
      {
        allocatormap_ = new std::map<std::string, SSTDesc_base*>();
      }

    add_to_map(name, descr, allocatormap_);
  }

  sstmac::sw::allocationstrategy::ptr
  SSTAllocatorFactory::get_allocator(std::string name, sim_parameters::ptr params, sstmac::hw::topology::ptr top)
  {

    if (allocatormap_->find(name) == allocatormap_->end())
      {
        std::cout << "allocator name: " << name << "\n";
        throw ssterror(
            "SSTAppFactory::get_allocator -- allocator was not found in allocator map, did you forget to call SSTRegisterAllocator?");
      }

    ptr_type::ptr p = (*allocatormap_)[name]->build(params);
    allocationstrategy::ptr cast = boost::dynamic_pointer_cast<allocationstrategy>(p);
    cast->set_topology(top);

    return cast;
  }

}
