/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/common/factories/app_factory.h>

namespace sstmac
{
  using namespace sstmac::sw;

  std::map<std::string, SSTDesc_base*>* SSTAppFactory::appmap_ = 0;

  void
  SSTAppFactory::register_app(std::string name, SSTDesc_base* descr)
  {
    if (appmap_ == 0)
      {
        appmap_ = new std::map<std::string, SSTDesc_base*>();
      }

    add_to_map(name, descr, appmap_);
  }

  sstmac::sw::app::ptr
  SSTAppFactory::get_app(std::string name, sim_parameters::ptr params)
  {

    if (appmap_->find(name) == appmap_->end())
      {
        std::cout << "app name: " << name << "\n";
        throw ssterror(
            "SSTAppFactory::get_app -- app was not found in app map, did you forget to call SSTRegisterApp?");
      }

    ptr_type::ptr p = (*appmap_)[name]->build(params);
    app::ptr cast = boost::dynamic_pointer_cast<app>(p);

    cast->init_libs(params);

    return cast;
  }

}
