/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_COMMON_FACTORIES_APP_FACTORY_H_INCLUDED
#define SSTMAC_COMMON_FACTORIES_APP_FACTORY_H_INCLUDED

#include <sstmac/common/factories/factory.h>
#include <sstmac/software/process/app.h>

#define SSTRegisterApp(appstr, appname, ...) static sstmac::SSTAppFactory_desc<appname> appname##_cd(appstr)

namespace sstmac
{

  class SSTAppFactory : public SSTFactory_base
  {
  private:
    static std::map<std::string, SSTDesc_base*>* appmap_;

  public:
    static void
    register_app(std::string name, SSTDesc_base* descr);

    static sstmac::sw::app::ptr
    get_app(std::string name, sim_parameters::ptr params);


  };

  template<class T>
    class SSTAppFactory_desc : public SSTDesc_base
    {

    public:
      SSTAppFactory_desc(const char* name)
      {
        std::string n(name);
        /** This should only ever be declared within
         the static initialization routine. There is therefore
         nothing unsafe about passing "this" out.  This object
         will be persisent throughout the entire simulation */
        SSTAppFactory::register_app(n, this);
      }

      ptr_type::ptr
      build(sim_parameters::ptr params)
      {
        if (!app_)
          {
            sstmac::sw::appid aid(0);
            sstmac::sw::taskid tid(0);
            sstmac::sw::softwareid sid(aid, tid);
            sstmac::sw::app::ptr application(T::construct(sid));
            app_ = application;
          }
        return app_;
      }

    protected:
      sstmac::sw::app::ptr app_;

    };

}

#endif
