/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMACRO_COMMON_FACTORIES_FACTORY_H_INCLUDED
#define SSTMACRO_COMMON_FACTORIES_FACTORY_H_INCLUDED

#include <sstmac/common/ptr_type.h>
#include <sstmac/common/basicstringtokenizer.h>
#include <deque>
#include <sstmac/common/sim_parameters.h>

namespace sstmac
{

  class SSTDesc_base
  {
  public:

    virtual ptr_type::ptr
    build(sim_parameters::ptr params) = 0;

  };

  class SSTFactory_base
  {
  public:
    static void
    add_to_map(std::string namestr, SSTDesc_base* desc,
        std::map<std::string, SSTDesc_base*> *m)
    {
      std::string space = "|";
      std::deque<std::string> tok;
      pst::BasicStringTokenizer::tokenize(namestr, tok, space);

      std::deque<std::string>::iterator it, end = tok.end();

      for (it = tok.begin(); it != end; it++)
        {
          std::string temp = *it;

          temp = trim_str(temp);

          (*m)[temp] = desc;

        }
    }

  };

}

#endif
