/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/common/factories/flower_factory.h>

namespace sstmac
{
  using namespace sstmac::hw;

  std::map<std::string, SSTDesc_base*>* SSTFlowerFactory::flowmap_ = 0;

  void
  SSTFlowerFactory::register_top(std::string name, SSTDesc_base* descr)
  {
    if (flowmap_ == 0)
      {
        flowmap_ = new std::map<std::string, SSTDesc_base*>();
      }

    add_to_map(name, descr, flowmap_);
  }

  sstmac::hw::flower::ptr
  SSTFlowerFactory::get_flower(std::string name, sim_parameters::ptr params)
  {

    if (flowmap_->find(name) == flowmap_->end())
      {
        std::cout << "flower name: " << name << "\n";
        throw ssterror(
            "SSTFlowerFactory::get_flower -- flower was not found in flower map, did you forget to call SSTRegisterFlower?");
      }

    ptr_type::ptr p = (*flowmap_)[name]->build(params);
    flower::ptr cast = boost::dynamic_pointer_cast<flower>(p);

    return cast;
  }

}
