/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/common/factories/indexer_factory.h>

namespace sstmac
{
  using namespace sstmac::sw;

  std::map<std::string, SSTDesc_base*>* SSTIndexerFactory::Indexermap_ = 0;

  void
  SSTIndexerFactory::register_Indexer(std::string name, SSTDesc_base* descr)
  {
    if (Indexermap_ == 0)
      {
        Indexermap_ = new std::map<std::string, SSTDesc_base*>();
      }

    add_to_map(name, descr, Indexermap_);
  }

  sstmac::sw::indexstrategy::ptr
  SSTIndexerFactory::get_Indexer(std::string name, sim_parameters::ptr params)
  {

    if (Indexermap_->find(name) == Indexermap_->end())
      {
        std::cout << "Indexer name: " << name << "\n";
        throw ssterror(
            "SSTAppFactory::get_Indexer -- Indexer was not found in Indexer map, did you forget to call SSTRegisterIndexer?");
      }

    ptr_type::ptr p = (*Indexermap_)[name]->build(params);
    indexstrategy::ptr cast = boost::dynamic_pointer_cast<indexstrategy>(p);


    return cast;
  }

}
