/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_COMMON_FACTORIES_Launch_FACTORY_H_INCLUDED
#define SSTMAC_COMMON_FACTORIES_Launch_FACTORY_H_INCLUDED

#include <sstmac/common/factories/factory.h>
#include <sstmac/software/services/launch/launcher.h>
#include <sstmac/software/services/launch/launchinfo.h>

#include <sstmac/hardware/network/topology/topology.h>

#define SSTRegisterLaunch(appstr, appname, ...) static sstmac::SSTLaunchFactory_desc<appname> appname##_cd(appstr);

namespace sstmac
{

  class SSTLaunchFactory : public SSTFactory_base
  {
  private:
    static std::map<std::string, SSTDesc_base*>* Launchmap_;

  public:
    static void
    register_Launch(std::string name, SSTDesc_base* descr);

    static sstmac::sw::launcher::ptr
    get_Launch(std::string name, sim_parameters::ptr params,
        sstmac::hw::topology::ptr top);

    static std::list<boost::intrusive_ptr<const sstmac::sw::launchinfo> >
    construct_launchers(sstmac::sim_parameters::ptr params);

    static void
    parse_aprun(const std::string& cmd, long& nproc, long& nproc_per_node,
        std::vector<int>& core_affinities, std::string& appname);

  };

  template<class T>
    class SSTLaunchFactory_desc : public SSTDesc_base
    {

    public:
      SSTLaunchFactory_desc(const char* name)
      {
        std::string n(name);
        /** This should only ever be declared within
         the static initialization routine. There is therefore
         nothing unsafe about passing "this" out.  This object
         will be persisent throughout the entire simulation */
        SSTLaunchFactory::register_Launch(n, this);
      }

      ptr_type::ptr
      build(sim_parameters::ptr params)
      {
        return T::construct(params);
      }

    };

}

#endif
