/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/common/factories/network_factory.h>


namespace sstmac
{
  using namespace sstmac::hw;

  std::map<std::string, SSTDesc_base*>* SSTNetworkFactory::netmap_ = 0;

  void
  SSTNetworkFactory::register_network(std::string name, SSTDesc_base* descr)
  {
    if(netmap_ == 0){
         netmap_ = new std::map<std::string, SSTDesc_base*>();
    }
    add_to_map(name, descr, netmap_);
  }

  sstmac::hw::interconnect::ptr
  SSTNetworkFactory::get_network(std::string name, sim_parameters::ptr params)
  {
    if(netmap_ == 0){
        throw ssterror("SSTNetworkFactory::get_network -- map was never created");
    }

    if (netmap_->find(name) == netmap_->end())
      {
        std::cout << "network name: " << name << "\n";
        throw ssterror(
            "SSTNetworkFactory::get_network -- network was not found in network map, did you forget to call SSTRegisterNetwork?");
      }

    ptr_type::ptr p = (*netmap_)[name]->build(params);
    interconnect::ptr cast = boost::dynamic_pointer_cast<interconnect>(p);


    if(!cast){
        throw ssterror("SSTNetworkFactory::get_network -- something is screwy, pointer didn't cast to interconnect");
    }

    return cast;
  }

}
