/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/common/factories/nic_factory.h>
#include <sstmac/hardware/network/congestion/interconnect.h>

namespace sstmac
{
  using namespace sstmac::hw;

  std::map<std::string, SSTDesc_base*>* SSTNicFactory::nicmap_ = 0;

  void
  SSTNicFactory::register_nic(std::string name, SSTDesc_base* descr)
  {
    if (nicmap_ == 0)
      {
        nicmap_ = new std::map<std::string, SSTDesc_base*>();
      }

    add_to_map(name, descr, nicmap_);
  }

  networkinterface::ptr
  SSTNicFactory::get_nic(std::string name, sim_parameters::ptr params, node::ptr n, boost::intrusive_ptr<interconnect> interconn)
  {

    if (nicmap_->find(name) == nicmap_->end())
      {
        std::cout << "nic name: " << name << "\n";
        throw ssterror(
            "SSTnicFactory::get_nic -- nic was not found in nic map, did you forget to call SSTRegisternic?");
      }

    ptr_type::ptr p = (*nicmap_)[name]->build(params);
    networkinterface::ptr cast = boost::dynamic_pointer_cast<networkinterface>(p);
    cast->init(n, interconn);
    return cast;
  }

}
