/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_COMMON_FACTORIES_nic_FACTORY_H_INCLUDED
#define SSTMAC_COMMON_FACTORIES_nic_FACTORY_H_INCLUDED

#include <sstmac/common/factories/factory.h>
#include <sstmac/hardware/nic/networkinterface.h>
#include <sstmac/hardware/node/node.h>

class sstmac::hw::interconnect;

#define SSTRegisterNic(appstr, appname, ...) static sstmac::SSTNicFactory_desc<appname> appname##_cd(appstr)

namespace sstmac
{

  class SSTNicFactory : public SSTFactory_base
  {
  private:
    static std::map<std::string, SSTDesc_base*>* nicmap_;

  public:
    static void
    register_nic(std::string name, SSTDesc_base* descr);

    static sstmac::hw::networkinterface::ptr
    get_nic(std::string name, sim_parameters::ptr params, sstmac::hw::node::ptr n, boost::intrusive_ptr<sstmac::hw::interconnect> interconn);

  };

  template<class T>
    class SSTNicFactory_desc : public SSTDesc_base
    {

    public:
      SSTNicFactory_desc(const char* name)
      {
        std::string n(name);
        /** This should only ever be declared within
         the static initialization routine. There is therefore
         nothing unsafe about passing "this" out.  This object
         will be persisent throughout the entire simulation */
        SSTNicFactory::register_nic(n, this);
      }

      ptr_type::ptr
      build(sim_parameters::ptr params)
      {

        return T::construct(params);
      }


    };

}

#endif
