/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/common/factories/node_factory.h>

namespace sstmac
{
  using namespace sstmac::hw;

  std::map<std::string, SSTDesc_base*>* SSTNodeFactory::nodemap_ = 0;

  void
  SSTNodeFactory::register_node(std::string name, SSTDesc_base* descr)
  {
    if (nodemap_ == 0)
      {
        nodemap_ = new std::map<std::string, SSTDesc_base*>();
      }

    add_to_map(name, descr, nodemap_);
  }

  node::ptr
  SSTNodeFactory::get_node(std::string name, sim_parameters::ptr params)
  {

    if (nodemap_->find(name) == nodemap_->end())
      {
        std::cout << "node name: " << name << "\n";
        throw ssterror(
            "SSTNodeFactory::get_node -- node was not found in node map, did you forget to call SSTRegisterNode?");
      }

    ptr_type::ptr p = (*nodemap_)[name]->build(params);
    node::ptr cast = boost::dynamic_pointer_cast<node>(p);

    return cast;
  }

}
