/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/common/factories/service_factory.h>
#include <sstmac/hardware/node/node.h>
#include <sstmac/software/services/service.h>

namespace sstmac
{
  using namespace sstmac::sw;

  std::map<std::string, SSTDesc_base*>* SSTServiceFactory::servmap_ = 0;

  void
  SSTServiceFactory::register_service(std::string name, SSTDesc_base* descr)
  {
    if (servmap_ == 0)
      {
        servmap_ = new std::map<std::string, SSTDesc_base*>();
      }
    add_to_map(name, descr, servmap_);
  }

  void
  SSTServiceFactory::start_services(sstmac::hw::node::ptr node,
      sim_parameters::ptr params)
  {

    if (servmap_ != 0)
      {
        std::map<std::string, SSTDesc_base*>::const_iterator it, end =
            servmap_->end();
        for (it = servmap_->begin(); it != end; ++it)
          {
            ptr_type::ptr p = it->second->build(params);
            service::ptr serv = boost::dynamic_pointer_cast<service>(p);
            node->add_service(serv);
          }
      }
  }

} //end namespace sstmac

