/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_COMMON_FACTORIES_SERVICE_FACTORY_H_INCLUDED
#define SSTMAC_COMMON_FACTORIES_SERVICE_FACTORY_H_INCLUDED

#include <sstmac/common/factories/factory.h>

#include <boost/intrusive_ptr.hpp>
#include <map>

#define SSTRegisterServ(servstr, appname, ...) static sstmac::sw::SSTServDesc<appname> appname##_cd(servstr)

namespace sstmac
{
  namespace hw
  {

    class node;
  }
}
namespace sstmac
{
  namespace sw
  {

    class service;

    class SSTServiceFactory : public SSTFactory_base
    {
    private:
      static std::map<std::string, SSTDesc_base*>* servmap_;

    public:
      static void
      register_service(std::string name, SSTDesc_base* descr);

      static void
      start_services(boost::intrusive_ptr<sstmac::hw::node> n, sim_parameters::ptr params);

    };

    template<class T>
      class SSTServDesc : public SSTDesc_base
      {

      public:
        SSTServDesc(const char* name)
        {
          std::string n(name);
          /** This should only ever be declared within
           the static initialization routine. There is therefore
           nothing unsafe about passing "this" out.  This object
           will be persisent throughout the entire simulation */
          SSTServiceFactory::register_service(n, this);
        }

        ptr_type::ptr
        build(sim_parameters::ptr params)
        {

          boost::intrusive_ptr<service> serv(T::construct());
          return serv;
        }

      };

  } //end namespace sstmac
} //end namespace sw

#endif
