/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/common/factories/topology_factory.h>

namespace sstmac
{
  using namespace sstmac::hw;

  std::map<std::string, SSTDesc_base*>* SSTTopologyFactory::topmap_ = 0;

  void
  SSTTopologyFactory::register_top(std::string name, SSTDesc_base* descr)
  {
    if (topmap_ == 0)
      {
        topmap_ = new std::map<std::string, SSTDesc_base*>();
      }
    add_to_map(name, descr, topmap_);
  }

  sstmac::hw::topology::ptr
  SSTTopologyFactory::get_topology(std::string name, sim_parameters::ptr params)
  {

    if (topmap_->find(name) == topmap_->end())
      {
        std::cout << "topology name: " << name << "\n";
        throw ssterror(
            "SSTTopologyFactory::get_topology -- topology was not found in topology map, did you forget to call SSTTopologyApp?");
      }

    ptr_type::ptr p = (*topmap_)[name]->build(params);
    topology::ptr cast = boost::dynamic_pointer_cast<topology>(p);

    return cast;
  }

}
