/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_COMMON_FACTORIES_TOPOLOGY_FACTORY_H_INCLUDED
#define SSTMAC_COMMON_FACTORIES_TOPOLOGY_FACTORY_H_INCLUDED

#include <sstmac/common/factories/factory.h>
#include <sstmac/hardware/network/topology/topology.h>

#define SSTRegisterTopology(appstr, appname, ...) static sstmac::SSTTopologyFactory_desc<appname> appname##_cd(appstr)

namespace sstmac
{

  class SSTTopologyFactory : public SSTFactory_base
  {
  private:
    static std::map<std::string, SSTDesc_base*>* topmap_;

  public:
    static void
    register_top(std::string name, SSTDesc_base* descr);

    static sstmac::hw::topology::ptr
    get_topology(std::string name, sim_parameters::ptr params);

  };

  template<class T>
    class SSTTopologyFactory_desc : public SSTDesc_base
    {

    public:
      SSTTopologyFactory_desc(const char* name)
      {
        std::string n(name);
        /** This should only ever be declared within
         the static initialization routine. There is therefore
         nothing unsafe about passing "this" out.  This object
         will be persisent throughout the entire simulation */
        SSTTopologyFactory::register_top(n, this);
      }

      ptr_type::ptr
      build(sim_parameters::ptr params)
      {
        return T::construct(params);
      }


    };

}

#endif
