#ifndef MALLOC_H
#define MALLOC_H

#include <vector>
#include <iostream>

#define FAST_MALLOC_INCREMENT 1024
#define INIT_FAST_MALLOC(x) \
    template<> std::vector<void*> FastMalloc<x>::allocations_(0, (void*) NULL); \
    template<> size_t FastMalloc<x>::allocation_increment_ = 0; \
    template<> size_t FastMalloc<x>::nmallocd_ = 0; \
    FastMallocDeleter<x> fast_malloc_##x##_deleter

template <class T>
class FastMalloc {

    public:
        static std::vector<void*> allocations_;

        static size_t allocation_increment_;

        static size_t nmallocd_;

        static void static_delete_finalize()
        {
            std::vector<void*>::iterator it = allocations_.begin(), end = allocations_.end();
            for ( ; it != end; ++it){
                ::free(*it);
            }
        }

        void* operator new(size_t size)
        {
            if (size != sizeof(T))
            {
                std::cerr << "Misaligned data dizes in FastMalloc. "
                          << "Probably the subclass hierarchy was not built properly\n";
                abort();
            }
            if (allocations_.empty())
            {
                if (nmallocd_ == allocation_increment_)
                {
                    allocation_increment_ = nmallocd_ == 0 ? 1024 : allocation_increment_ * 2;
                    allocations_.reserve(allocation_increment_);
                }
                void* ptr = ::malloc(size);
                ++nmallocd_;
                return ptr;
            }
            else
            {
                void* ptr = allocations_.back();
                allocations_.pop_back();
                return ptr;
            }
        }

        void operator delete(void* ptr)
        {
            allocations_.push_back(ptr);
        }

};


template <class T>
class FastMallocDeleter {
    public:
        ~FastMallocDeleter(){
            FastMalloc<T>::static_delete_finalize();
        }
};

#endif // MALLOC_H
