/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/hardware/memory/simple_memorymodel.h>

namespace sstmac
{
  namespace hw
  {

    simple_memorymodel::simple_memorymodel(const node::ptr& parent_node,
                                           double mem_bw, timestamp mem_lat,
                                           eventhandler::ptr whendone, long nodeid)
      : memorymodel(parent_node, nodeid),
        done_(whendone)
    {
      link_ = link::construct(mem_bw, mem_lat);
    }

    simple_memorymodel::~simple_memorymodel()
    {
      link_ = link::ptr();
    }

    void
    simple_memorymodel::handle(const sst_message::ptr &msg)
    {
      if(msg->get_type() == sst_message::FAILURE){
          failure_occurred(msg, ptr(this));
          return;
      }
      link_->access_done();

      msg->add_callback(done_);


      done_->handle(msg);
    }

    void
    simple_memorymodel::finish()
    {
        done_ = 0;
        memorymodel::finish();
    }

    void
    simple_memorymodel::read(const sst_message::ptr &msg)
    {
        use_me();

        SSTMAC_DEBUG << "simple_memorymodel: doing access of " << msg->get_byte_length() << " \n";

        mem_bytes_->record(msg->get_byte_length());
        timestamp delta_t = link_->new_access(now(), msg->get_byte_length());
        delta_t += msg->get_event_time();
        schedule(now() + delta_t, ptr(this), msg);
    }

    void
    simple_memorymodel::write(const sst_message::ptr &msg)
    {
        /** just treat as a read */
        read(msg);
    }

    void
    simple_memorymodel::nic_read(const sst_message::ptr &msg)
    {
        read(msg);
    }

    void
    simple_memorymodel::nic_write(const sst_message::ptr &msg)
    {
        write(msg);
    }

  }
} /* namespace sstmac */
