/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */
#ifndef SSTMAC_HARDWARE_NETWORK_MULTI_DIVIDER_H_
#define SSTMAC_HARDWARE_NETWORK_MULTI_DIVIDER_H_

#include <sstmac/common/messages/sst_message.h>
#include <sstmac/common/sim_parameters.h>
#include <sstmac/common/errors.h>

namespace sstmac
{
  namespace hw
  {
    class divider : public ptr_type
    {
    protected:
      int num_;

      divider(int n) :
        num_(n)
      {

      }

    public:
      typedef boost::intrusive_ptr<divider> ptr;

      virtual
      ~divider()
      {
      }

      virtual std::string
      to_string() const
      {
        return "divider";
      }

      virtual int
      which(sst_message::ptr msg) = 0;

    };

    class size_divider : public divider
    {

    protected:

      long thresh_;

      size_divider(int n, long thresh) :
        divider(n), thresh_(thresh)
      {

      }

    public:
      typedef boost::intrusive_ptr<size_divider> ptr;

      static ptr
      construct(int n, sim_parameters::ptr params)
      {
        long thresh = params->get_long_param(
            "multi_sizedivider_threshold");
        if (n > 2)
          {
            throw ssterror(
                "size_divider::construct - i don't know what to do with more than 2 networks");
          }
        return ptr(new size_divider(n, thresh));
      }

      virtual int
      which(sst_message::ptr msg)
      {
        if (msg->get_byte_length() < thresh_)
          {
            return 1;
          }
        else
          {
            return 2;
          }

      }
    };
  }
}

#endif
