/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_HARDWARE_NETWORK_SWITCHES_ENTRYEXIT_H_INCLUDED
#define SSTMAC_HARDWARE_NETWORK_SWITCHES_ENTRYEXIT_H_INCLUDED

#include <sstmac/common/eventhandler.h>
#include <boost/unordered_map.hpp>
namespace sstmac
{
  namespace hw
  {

    class injector : public eventhandler
    {

    protected:

      boost::unordered_map<nodeaddress::ptr, eventhandler::ptr> sendhandle_;


    public:
      typedef boost::intrusive_ptr<injector> ptr;

      virtual std::string
      to_string() const
      {
        return "injector";
      }

      virtual
      ~injector()
      {
      }

      void
      set_sendhandler(const eventhandler::ptr &hand, nodeaddress::ptr addr)
      {
        sendhandle_[addr] = hand;
      }


    };

    class ejector : public eventhandler
    {
    protected:
      boost::unordered_map<nodeaddress::ptr, eventhandler::ptr> recvhandles_;

    public:
      typedef boost::intrusive_ptr<ejector> ptr;

      virtual std::string
      to_string() const
      {
        return "ejector";
      }

      virtual
      ~ejector()
      {
      }

      void
      set_recvhandler(const eventhandler::ptr &hand, nodeaddress::ptr addr)
      {
        recvhandles_[addr] = hand;
      }

    };

  }
}

#endif
