/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_HARDWARE_NETWORK_SWITCHES_NETWORKSWITCH_H_INCLUDED
#define SSTMAC_HARDWARE_NETWORK_SWITCHES_NETWORKSWITCH_H_INCLUDED

#include <sstmac/common/eventmodule.h>
#include <sstmac/common/sim_parameters.h>
#include <sstmac/hardware/network/switches/routing/router.h>
#include <sstmac/hardware/network/switches/entryexit.h>

#include <sstmac/common/vis/vis.h>

namespace sstmac
{
  namespace hw
  {
    class networkswitch : public eventmodule, public vis::vis_comp
    {

    protected:

      networkswitch()
      {
        if (!dbg_.is_real())
          {
            dbg_ = debug("<debug> networkswitch | network");
            logger::register_logger(&dbg_);
          }

      }

      void
      init(nodeaddress::ptr me, router::ptr route)
      {
        my_addr_ = me;
        router_ = route;
      }

      nodeaddress::ptr my_addr_;
      router::ptr router_;

      static debug dbg_;

      //      dim            dir    switch
      std::map<int, std::map<int, eventhandler::ptr> > outports_;

      //this map tracks which connections have multiple
      //  link going to the same place
      std::map<int, std::map<int, int> > redund_;

      //      dim            dir  bandwidth
      std::map<int, std::map<int, double> > portbw_;

      //      dim            dir    vis obj
      std::map<int, std::map<int, vis::vis_obj::ptr> > linkvis_;

      std::map<int, injector::ptr> ij_;
      std::map<int, ejector::ptr> ej_;

      virtual injector::ptr
      get_new_injector(sim_parameters::ptr params) = 0;

      virtual ejector::ptr
      get_new_ejector(sim_parameters::ptr params) = 0;

    public:
      typedef boost::intrusive_ptr<networkswitch> ptr;

      enum PORTS
      {
        EJECT_PORT = -1
      };

      virtual void
      connect(int dim, int dir, networkswitch::ptr sw, double bw, int redund = 1)
      {
        outports_[dim][dir] = sw;
        portbw_[dim][dir] = bw;

        redund_[dim][dir] = redund;
      }

      nodeaddress::ptr
      get_addr() const
      {
        return my_addr_;
      }

      router::ptr
      get_router() const
      {
        return router_;
      }
      virtual void
      finish()
      {
      }

      virtual std::string
      to_string() const
      {
        return "network_switch";
      }

      virtual void
      set_link_vis(vis::vis_obj::ptr vo, int dim, int dir)
      {
        linkvis_[dim][dir] = vo;
      }

      virtual void
      initialize(sim_parameters::ptr params, int nps) = 0;

      virtual ptr
      clone(router::ptr route, nodeaddress::ptr addr) = 0;

      injector::ptr
      get_injector(sim_parameters::ptr params, int c);

      ejector::ptr
      get_ejector(sim_parameters::ptr params, int c);

      virtual
      ~networkswitch()
      {
      }

    };
  }
}

#endif
