/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_HARDWARE_NETWORK_SWTICHES_PACKET_PACKET_ROUTING_INFO_H_INCLUDED
#define SSTMAC_HARDWARE_NETWORK_SWTICHES_PACKET_PACKET_ROUTING_INFO_H_INCLUDED

#include <sstmac/hardware/network/switches/routing/routing_info.h>

namespace sstmac
{
  namespace hw
  {

    class packet_routing_info : public congestion_routing_info
    {
    protected:
      //      dim            dir   relative
      std::map<int, std::map<int, double> > congestion_;

      packet_routing_info(nodeaddress::ptr to) :
        congestion_routing_info(to)
      {

      }

    public:
      typedef boost::intrusive_ptr<packet_routing_info> ptr;

      static ptr
      construct(nodeaddress::ptr to)
      {
        return ptr(new packet_routing_info(to));
      }

      virtual
      ~packet_routing_info()
      {
      }

      virtual std::string
      to_string() const
      {
        return "packet_routing_info";
      }

      void
      set_congestion(
          const std::map<int, std::map<int, std::map<int, timestamp> > > &finishtimes,
          timestamp maxtime, timestamp now)
      {

        std::map<int, std::map<int, std::map<int, timestamp> > >::const_iterator
            it1, end1 = finishtimes.end();
        for (it1 = finishtimes.begin(); it1 != end1; it1++)
          {

            std::map<int, std::map<int, timestamp> >::const_iterator it2, end2 =
                it1->second.end();
            for (it2 = it1->second.begin(); it2 != end2; it2++)
              {
                std::map<int, timestamp>::const_iterator it3, end3 =
                    it2->second.end();
                double avg = 0;
                int num = 0;
                for (it3 = it2->second.begin(); it3 != end3; it3++)
                  {
                    if (it3->second.sec() - now.sec() >= 0)
                      {
                        avg += it3->second.sec() - now.sec();
                        num++;
                      }
                  }
                if (num > 0)
                  {
                    avg /= (double) num;

                    if (maxtime.sec() == 0)
                      {
                        congestion_[it1->first][it2->first] = avg;
                      }
                    else
                      {
                        congestion_[it1->first][it2->first] = avg
                            / maxtime.sec();
                      }
                  }
                else
                  {
                    congestion_[it1->first][it2->first] = 0;
                  }

              }
          }

      }

      virtual void
      get_min_congestion_port(int &dim, int &dir)
      {
        dim = -1;
        dir = -1;
        double min = 100000000;
        std::map<int, std::map<int, double> >::iterator it, end =
            congestion_.end();
        for (it = congestion_.begin(); it != end; it++)
          {
            std::map<int, double>::iterator it2, end2 = it->second.end();
            for (it2 = it->second.begin(); it2 != end2; it2++)
              {
                if (it2->second < min)
                  {
                    min = it2->second;
                    dim = it->first;
                    dir = it2->first;
                  }
              }
          }
      }

      virtual std::map<int, std::map<int, double> >
      get_congestion_info()
      {
        return congestion_;
      }

    };

  }
}
#endif
