/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_HARDWARE_NETWORK_SWTICHES_ROUTING_butterflyrouter_H_INCLUDED
#define SSTMAC_HARDWARE_NETWORK_SWTICHES_ROUTING_butterflyrouter_H_INCLUDED

#include <sstmac/hardware/network/switches/routing/router.h>
#include <vector>
#include <sstmac/common/messages/sst_message.h>
#include <sstmac/hardware/network/switches/circuit/circuit_router.h>

namespace sstmac
{
  namespace hw
  {

    class butterflyrouter : public virtual router, public virtual circuit_router
    {
    public:
      typedef boost::intrusive_ptr<butterflyrouter> ptr;

      static ptr
      construct(nodeaddress::ptr me, int n, int r, int s)
      {
        return ptr(new butterflyrouter(me, n, r, s));
      }

      virtual void
      route(routing_info::ptr to, int &dim, int &dir);

      virtual int
      convert_to_outport(int dim, int dir);

      virtual void
      convert_to_dimdir(int outport, int &dim, int&dir);

      virtual
      ~butterflyrouter()
      {
      }

      virtual void
      path_is_good(nodeaddress::ptr goingto, int fromport, int toport)
      {
      }

      virtual void
      path_teardown(int fromport, int toport)
      {
      }

      virtual int
      get_inport(nodeaddress::ptr from, int dim, int dir);

      virtual bool
      is_fixed_routing() const
      {
        return true;
      }

      virtual std::string
      to_string() const
      {
        return "butterflyrouter";
      }

    protected:
      int n_;
      int r_;
      int stages_;
      int numperstage_;

      std::map<int, int> inports_;
      int incnt_;

      butterflyrouter(nodeaddress::ptr me, int n, int r, int s) :
        router(me, r), n_(n), r_(r), stages_(s), incnt_(0)
      {
        numperstage_ = (n_ / r_);
      }

    };

  }
}
#endif
