/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_HARDWARE_NETWORK_SWTICHES_ROUTING_closrouter_H_INCLUDED
#define SSTMAC_HARDWARE_NETWORK_SWTICHES_ROUTING_closrouter_H_INCLUDED

#include <sstmac/hardware/network/switches/routing/router.h>
#include <vector>
#include <sstmac/common/messages/sst_message.h>
#include <sstmac/hardware/network/switches/circuit/circuit_router.h>

namespace sstmac
{
  namespace hw
  {

    class closrouter : public virtual router, public virtual circuit_router
    {
    public:
      typedef boost::intrusive_ptr<closrouter> ptr;

      static ptr
      construct(nodeaddress::ptr me, int n, int k, int m)
      {
        return ptr(new closrouter(me, n, k, m));
      }

      virtual void
      route(routing_info::ptr to, int &dim, int &dir);

      virtual int
      convert_to_outport(int dim, int dir);

      virtual void
      convert_to_dimdir(int outport, int &dim, int&dir);

      virtual
      ~closrouter()
      {
      }

      virtual int
      get_inport(nodeaddress::ptr from, int dim, int dir);

      virtual bool
      is_fixed_routing() const;

      virtual void
      path_is_good(nodeaddress::ptr goingto, int fromport, int toport);

      virtual void
      path_teardown(int fromport, int toport);

      virtual std::string
      to_string() const
      {
        return "closrouter";
      }

    protected:
      int n_;
      int k_;
      int m_;

      int cnt_;

      //keep track of the middle stage allocation


      static std::map<int, int> middlealloc_;

      //number of channels taken up
      static std::map<int, int> inchannelsalloc_;
      static std::map<int, int> outchannelsalloc_;

      closrouter(nodeaddress::ptr me, int n, int k, int m) :
        router(me, n), n_(n), k_(k), m_(m), cnt_(0)
      {
        for (int i = 0; i < k; i++)
          {
            for (int j = 0; j < m; j++)
              {
                int index = i * m + j;
                inchannelsalloc_[index] = 0;
                outchannelsalloc_[index] = 0;
              }
          }
      }

    };

  }
}
#endif
