/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_HARDWARE_NETWORK_SWITCHES_ROUTING_DRAGONFLYROUTER_H_INCLUDED
#define SSTMAC_HARDWARE_NETWORK_SWITCHES_ROUTING_DRAGONFLYROUTER_H_INCLUDED

#include <sstmac/hardware/network/switches/routing/router.h>

namespace sstmac
{
  namespace hw
  {

    class dragonflyrouter : public router
    {

    protected:
      dragonflyrouter(nodeaddress::ptr m, int npr, int numx, int numy, int numg) :
        router(m, npr), numX_(numx), numY_(numy), numG_(numg)
      {

       get_coords(m->unique_id(), myX_, myY_, myG_);
      }

    public:
      typedef boost::intrusive_ptr<dragonflyrouter> ptr;

      static ptr
      construct(nodeaddress::ptr m, int npr, int numx, int numy, int numg)
      {
        return ptr(new dragonflyrouter(m, npr, numx, numy, numg));
      }

      virtual void
      route(routing_info::ptr to, int &dim, int &dir);

      virtual int
      convert_to_outport(int dim, int dir);

      virtual void
      convert_to_dimdir(int outport, int &dim, int&dir);

      virtual int
      get_inport(nodeaddress::ptr from, int dim, int dir);

    protected:

      void
      get_coords(long uid, int &x, int &y, int &g)
      {
        x = uid % numX_;
        y = (uid / numX_) % numY_;
        g = uid / (numX_ * numY_);
      }

      int numX_;
      int numY_;
      int numG_;

      int myX_;
      int myY_;
      int myG_;

    };

  }
}
#endif
