/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_BACKENDS_OMNETPP_NETWORKMODELS_ROUTING_FLATTENEDBFLYROUTER_H_INCLUDED
#define SSTMAC_BACKENDS_OMNETPP_NETWORKMODELS_ROUTING_FLATTENEDBFLYROUTER_H_INCLUDED

#include <sstmac/hardware/network/switches/routing/torusrouter.h>

namespace sstmac
{
  namespace hw
  {

    class flattenedbflyrouter : public torusrouter
    {
    public:
      typedef boost::intrusive_ptr<flattenedbflyrouter> ptr;

      static ptr
      construct(nodeaddress::ptr me, int n, std::vector<int> dims)
      {
        return ptr(new flattenedbflyrouter(me, n, dims));
      }

      virtual void
      route(routing_info::ptr to, int &dim, int &dir);

      virtual int
      convert_to_outport(int dim, int dir);

      virtual void
      convert_to_dimdir(int outport, int &dim, int&dir);

      virtual int
      get_inport(nodeaddress::ptr from, int dim, int dir);

      virtual bool
      is_fixed_routing() const
      {
        return true;
      }

      virtual std::string
      to_string() const
      {
        return "flattenedbutterflyrouter";
      }

    protected:


      flattenedbflyrouter(nodeaddress::ptr me, int n, std::vector<int> dims) :
        torusrouter(me, n, dims)
      {

      }



    };

  }
}
#endif
