/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_HARDWARE_NETWORK_ROUTING_ROUTER_H_INCLUDED
#define SSTMAC_HARDWARE_NETWORK_ROUTING_ROUTER_H_INCLUDED

#include <sstmac/common/debug.h>
#include <sstmac/common/nodeaddress.h>
#include <sstmac/common/ptr_type.h>
#include <sstmac/hardware/network/switches/routing/routing_info.h>

namespace sstmac
{
  namespace hw
  {

    class router : public virtual ptr_type
    {
    public:
      typedef boost::intrusive_ptr<router> ptr;

      virtual std::string
      to_string() const
      {
        return "router";
      }

      virtual
      ~router()
      {
      }

      virtual void
      route(routing_info::ptr to, int &dim, int &dir) = 0;

      virtual int
      convert_to_outport(int dim, int dir) = 0;

      virtual void
      convert_to_dimdir(int outport, int &dim, int&dir) = 0;

      enum DIRECTION
      {
        POS, NEG, UP, DOWN, EAST, WEST, NORTH, SOUTH, EJECT = -1
      };

      int
      conc() const
      {
        return conc_;
      }

      virtual int
      get_inport(nodeaddress::ptr from, int dim, int dir) = 0;

      virtual bool
      is_fixed_routing() const
      {
        return false;
      }

      virtual void
      set_inport(nodeaddress::ptr from, int port)
      {
      }

    protected:
      nodeaddress::ptr me_;

      int conc_;

      static sstmac::debug dbg_;

      router(nodeaddress::ptr m, int npr)
      {
        me_ = m;
        conc_ = npr;

        if (!dbg_.is_real())
          {
            dbg_ = debug("<debug> router | network");
            logger::register_logger(&dbg_);
          }

      }

      nodeaddress::ptr
      convert_to_network_addr(nodeaddress::ptr n)
      {
        long uid = n->unique_id() / conc_;
        return nodeaddress::convert_from_uniqueid(uid);

      }

      DIRECTION
      get_opp_dir(int dir)
      {

        switch (dir)
          {
        case POS:
          return NEG;

        case NEG:
          return POS;

        case UP:
          return DOWN;

        case DOWN:
          return UP;

        case EAST:
          return WEST;

        case WEST:
          return EAST;

        case NORTH:
          return SOUTH;

        case SOUTH:
          return NORTH;
          }
      }
    };

  }
}
#endif
