/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_HARDWARE_NETWORK_ROUTING_ROUTING_INFO_H_INCLUDED
#define SSTMAC_HARDWARE_NETWORK_ROUTING_ROUTING_INFO_H_INCLUDED

#include <sstmac/common/ptr_type.h>

namespace sstmac
{
  namespace hw
  {

    class routing_info : public ptr_type
    {
    protected:
      nodeaddress::ptr to_;
      bool is_min_;

      routing_info(nodeaddress::ptr to) :
        to_(to), is_min_(true)
      {

      }

    public:
      typedef boost::intrusive_ptr<routing_info> ptr;

      static ptr
      construct(nodeaddress::ptr to)
      {
        return ptr(new routing_info(to));
      }

      virtual
      ~routing_info()
      {
      }

      virtual std::string
      to_string() const
      {
        return "routing_info";
      }

      nodeaddress::ptr
      to()
      {
        return to_;
      }

      virtual bool
      is_min() const
      {
        return is_min_;
      }

      virtual void
      set_min(bool b)
      {
        is_min_ = b;
      }
    };

    class congestion_routing_info : public routing_info
    {
    protected:
      congestion_routing_info(nodeaddress::ptr to) :
        routing_info(to)
      {

      }
    public:
      typedef boost::intrusive_ptr<congestion_routing_info> ptr;

      virtual
      ~congestion_routing_info()
      {
      }

      virtual std::string
      to_string() const
      {
        return "congestion_routing_info";
      }

      virtual std::map<int, std::map<int, double> >
      get_congestion_info() = 0;

      virtual void
      get_min_congestion_port(int &dim, int &dir) = 0;

    };

  }
}
#endif
