/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/hardware/network/switches/routing/torusrouter.h>
#include <sstmac/hardware/network/switches/networkswitch.h>
namespace sstmac
{
  namespace hw
  {

    std::vector<int>
    torusrouter::get_coords(long uid)
    {
      long div = 1;
      std::vector<int> ret(dims_.size());
      for (int i = 0; i < dims_.size(); i++)
        {
          ret[i] = (uid / div) % dims_[i];
          div *= dims_[i];
        }
      return ret;
    }

    long
    torusrouter::get_uid(std::vector<int> coords)
    {
      long ret = 0;
      long mult = 1;
      for (int i = 0; i < dims_.size(); i++)
        {
          ret += coords[i] * mult;
          mult *= dims_[i];
        }
      return ret;
    }

    void
    torusrouter::route(routing_info::ptr to, int &dim, int &dir)
    {

      nodeaddress::ptr netaddr = convert_to_network_addr(to->to());
      std::vector<int> coords = get_coords(netaddr->unique_id());
      SSTMAC_DEBUG << "flattenedbflyrouter(" << me_->to_string()
            << "): heading to " << to->to()->to_string() << " at net addr "
            << netaddr->to_string() << "\n";

      for (int i = 0; i < dims_.size(); i++)
        {
          if (coords[i] != coords_[i])
            {
              dim = i;
            }
          else
            {
              continue;
            }

          if (coords[i] > coords_[i] && fabs(
              coords_[i] + (dims_[i] - coords[i])) > dims_[i] / 2 || coords[i]
              < coords_[i] && fabs(coords[i] + (dims_[i] - coords_[i]))
              <= dims_[i] / 2)
            {

              dir = POS;
            }
          else if (coords[i] < coords_[i] && fabs(
              coords[i] + (dims_[i] - coords_[i])) > dims_[i] / 2 || coords[i]
              > coords_[i] && fabs(coords_[i] + (dims_[i] - coords[i]))
              <= dims_[i] / 2)
            {

              dir = NEG;

            }
          else
            {
              throw ssterror("flattenedbflyrouter::route -- i'm confused");
            }
          SSTMAC_DEBUG << "returning dim " << dim << " and dir " << dir << "\n";
          return;

        }

      dim = networkswitch::EJECT_PORT;
      dir = to->to()->unique_id() % conc_;
      return;
    }

    int
    torusrouter::convert_to_outport(int dim, int dir)
    {
      if (dim == networkswitch::EJECT_PORT)
        {
          return -(1+dir);
        }
      return dim * 2 + (dir == POS ? 0 : 1);

    }

    void
    torusrouter::convert_to_dimdir(int outport, int &dim, int&dir)
    {
      if (outport < 0)
        {
          dim = networkswitch::EJECT_PORT;
          dir = (-outport)-1;
        }
      else
        {
          dim = outport / 2;
          dir = (outport % 2 == 0 ? POS : NEG);
        }
    }

    int
    torusrouter::get_inport(nodeaddress::ptr from, int dim, int dir)
    {
      return convert_to_outport(dim, get_opp_dir(dir));
    }

  }
}
