/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_HARDWARE_NETWORK_SWTICHES_ROUTING_TORUSROUTER_H_INCLUDED
#define SSTMAC_HARDWARE_NETWORK_SWTICHES_ROUTING_TORUSROUTER_H_INCLUDED

#include <sstmac/hardware/network/switches/routing/router.h>
#include <vector>
#include <sstmac/common/messages/sst_message.h>

namespace sstmac
{
  namespace hw
  {

    class torusrouter : public router
    {
    public:
      typedef boost::intrusive_ptr<torusrouter> ptr;

      static ptr
      construct(nodeaddress::ptr m, int npr, std::vector<int> dims)
      {
        return ptr(new torusrouter(m, npr, dims));
      }

      virtual void
      route(routing_info::ptr to, int &dim, int &dir);

      virtual int
      convert_to_outport(int dim, int dir);

      virtual void
      convert_to_dimdir(int outport, int &dim, int&dir);

      virtual
      ~torusrouter()
      {
      }


      virtual int
      get_inport(nodeaddress::ptr from, int dim, int dir);

    protected:
      std::vector<int> dims_;
      std::vector<int> coords_;

      torusrouter(nodeaddress::ptr m, int npr, std::vector<int> dims) :
        router(m, npr), dims_(dims)
      {

        //  SSTMAC_DEBUG << "router rack id = " << me->rack() << "; coord = ("
        //       << myX << ", " << myY << ", " << myZ << ")\n";
        coords_ = get_coords(m->unique_id());
      }

      std::vector<int>
      get_coords(long i);

      long
      get_uid(std::vector<int> v);
    };

  }
}
#endif
