/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_HARDWARE_NETWORK_SWITCHES_SWITCHINTERCONNECT_H_INCLUDED
#define SSTMAC_HARDWARE_NETWORK_SWITCHES_SWITCHINTERCONNECT_H_INCLUDED

#include <sstmac/hardware/network/congestion/interconnect.h>
#include <sstmac/hardware/network/switches/networkswitch.h>
#include <sstmac/hardware/network/switches/entryexit.h>
#include <sstmac/hardware/network/switches/switchtopology.h>
#include <sstmac/hardware/network/topology/fake_topology.h>

#include <boost/unordered_map.hpp>
#include <sstmac/common/vis/vis.h>

#include <sstmac/common/driver_util.h>

namespace sstmac
{
  namespace hw
  {

    class switchinterconnect : public virtual interconnect,
        public virtual vis::vis_display
    {

    protected:

      switchinterconnect(topology::ptr top, sim_parameters::ptr params,
          vis::vis_engine::ptr veng);

      switchtopology::maptype switches_;
      switchtopology::injmaptype injector_;
      switchtopology::ejmaptype ejector_;

      switchtopology::ptr switchtop_;

    public:
      typedef boost::intrusive_ptr<switchinterconnect> ptr;

      virtual std::string
      to_string() const
      {
        return "switchinterconnect";
      }

      static ptr
      construct(sim_parameters::ptr params, topology::ptr top)
      {
        return ptr(
            new switchinterconnect(top, params, construct_vis_engine(params)));
      }

      virtual
      ~switchinterconnect()
      {

      }

      virtual void
      finish();

      virtual int64_t
      slots() const
      {
        return injector_.size();
      }

      virtual void
      set_endpoint(nodeaddress::ptr addr,
          boost::intrusive_ptr<eventhandler> nic)
      {
        interconnect::set_endpoint(addr, nic);
        SSTMAC_DEBUG << "switchinterconnect: setting eventhandler for addr "
              << addr->to_string() << "\n";
        injector_[addr]->set_sendhandler(nic, addr);
        ejector_[addr]->set_recvhandler(nic, addr);
      }

      /// Send the given message at the given time.
      /// Notify the sender node when the send is complete
      /// and notify the receiver node when receive needs to be performed.
      virtual void
      send(const timestamp &start_time, const sst_message::ptr &msg);

      virtual void
      set_eventmanager(const eventmanager::ptr &m);

      virtual vis::vis_topology::vismaptype
      vis_get_nodes();

      virtual vis::vis_topology::ptr
      vis_get_topology();
    };

  }
}

#endif
