/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_HARDWARE_NETWORK_SWITCHES_SWITCHTOPOLOGY_H_INCLUDED
#define SSTMAC_HARDWARE_NETWORK_SWITCHES_SWITCHTOPOLOGY_H_INCLUDED

#include <sstmac/common/ptr_type.h>
#include <sstmac/common/sstmac-config.h>
#include <sstmac/hardware/network/switches/networkswitch.h>
#include <sstmac/hardware/network/switches/entryexit.h>
#include <sstmac/hardware/network/topology/topology.h>

#include <sstmac/common/vis/vis.h>

namespace sstmac
{
  namespace hw
  {
    /**
     * This class is an interface for topologies which support
     * the switch network model
     */
    class switchtopology : virtual public topology,
        public virtual vis::vis_topology
    {

    public:
      typedef boost::intrusive_ptr<switchtopology> ptr;

      typedef boost::unordered_map<nodeaddress::ptr, networkswitch::ptr>
          maptype;
      typedef boost::unordered_map<nodeaddress::ptr, injector::ptr> injmaptype;
      typedef boost::unordered_map<nodeaddress::ptr, ejector::ptr> ejmaptype;

      virtual void
      connect_switches(maptype &switches, sim_parameters::ptr params,
          std::string linkbwname = "network_bandwidth_link") = 0;

      virtual router::ptr
      create_router(nodeaddress::ptr addr, sim_parameters::ptr params) = 0;

      virtual int
      radix() const = 0;

      virtual int64_t
      n() const
      {
        return topology_size();
      }

      virtual int64_t
      topology_size() const = 0;

      virtual void
      set_ejectors(long num, int nps, maptype switches, ejmaptype &ejectors,
          sim_parameters::ptr params)
      {
        for (long i = 0; i < num * nps; i++)
          {

            nodeaddress::ptr nodeaddr = nodeaddress::convert_from_uniqueid(i);

            nodeaddress::ptr swaddr = nodeaddress::convert_from_uniqueid(
                (i / nps));

            ejectors[nodeaddr] = switches[swaddr]->get_ejector(params, i % nps);

          }
      }

      virtual void
      display_nodes(vismaptype &switches, vis::vis_engine::ptr eng,
          std::list<vis::vis_obj::ptr> &objs)
      {

      }

      virtual
      ~switchtopology()
      {
      }

      virtual std::string
      to_string() const
      {
        return "switchtopology";
      }
    };
  }
}

#endif
