/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */
#include <sstmac/hardware/network/topology/dragonfly.h>
#include <sstmac/hardware/network/switches/routing/dragonflyrouter.h>
#include <math.h>
#include <sstmac/common/driver_util.h>
#include <sstmac/common/factories/topology_factory.h>


namespace sstmac
{
  namespace hw
  {
    SSTRegisterTopology("dragonfly | dfly", dragonfly);


    using namespace std;

    dragonfly::dragonfly(long x, long y, long g, int nps) :
      x_(x), y_(y), g_(g), nps_(nps)
    {

    }

    dragonfly::ptr
    dragonfly::construct(sim_parameters::ptr params)
    {
      bool errorflag = false;
      std::vector<int> args = get_intvec(
          params->get_param("topology_geometry").c_str(), errorflag);
      int nps = params->get_optional_int_param("network_nodes_per_switch", 1);
      return ptr(new dragonfly(args[0], args[1], args[2], nps));
    }

    void
    dragonfly::get_coords(long uid, int &x, int &y, int &g)
    {
      x = uid % x_;
      y = (uid / x_) % y_;
      g = uid / (x_ * y_);
    }

    long
    dragonfly::get_uid(int x, int y, int g)
    {
      return g * (x_ * y_) + y * (x_) + x;
    }

    std::string
    dragonfly::coord_string(int x, int y, int g)
    {
      std::stringstream ss;
      ss << "(" << x << ", " << y << ", " << g << ")";

      return ss.str();
    }

    /// coordinates go x,y - group
    void
    dragonfly::connect_switches(switchtopology::maptype &switches,
        sim_parameters::ptr params, std::string linkbwname)
    {
      /* double linkbw = params->get_double_param("network_bandwidth_link");
       for (long i = 0; i < switches.size(); i++)
       {
       nodeaddress::ptr me = nodeaddress::convert_from_uniqueid(i);
       int myx;
       int myy;
       int myg;
       get_coords(i, myx, myy, myg);

       for (int x = 0; x < x_; x++)
       {
       if (x != myx)
       {

       nodeaddress::ptr them = nodeaddress::convert_from_uniqueid(
       get_uid(x, myy, myg));
       SSTMAC_DEBUG << "node " << me->to_string() << " "
       << coord_string(myx, myy, myg)
       << " connecting to node " << them->to_string() << " "
       << coord_string(x, myy, myg) << "\n";
       switches[me]->connect(0, x, switches[them]);
       }
       }

       for (int y = 0; y < y_; y++)
       {
       if (y != myy)
       {

       nodeaddress::ptr them = nodeaddress::convert_from_uniqueid(
       get_uid(myx, y, myg));
       SSTMAC_DEBUG << "node " << me->to_string() << " "
       << coord_string(myx, myy, myg)
       << " connecting to node " << them->to_string() << " "
       << coord_string(myx, y, myg) << "\n";
       switches[me]->connect(1, y, switches[them]);
       }
       }

       //each switch connects to a max of 5 different groups
       int gspace = max(1, g_ / 5);
       int myid = myx + myy * x_;
       int gset = myid % g_;
       for (int g = 0; g < 5; g++)
       {

       int gid = (gset + gspace * g) % g_;
       if (gid != myg)
       {
       long uid = get_uid(myx, myy, gid);
       nodeaddress::ptr them = nodeaddress::convert_from_uniqueid(
       uid);

       SSTMAC_DEBUG << "node " << me->to_string() << " "
       << coord_string(myx, myy, myg)
       << " connecting to node " << them->to_string() << " "
       << coord_string(myx, myy, gid) << "\n";
       switches[me]->connect(2, gid, switches[them]);
       }

       }

       }*/

      throw ssterror("don't use dragonfly, use grouped topology");

    }

    router::ptr
    dragonfly::create_router(nodeaddress::ptr addr, sim_parameters::ptr params)
    {

      router::ptr ret = dragonflyrouter::construct(addr, nps_, x_, y_, g_);

      return ret;
    }

  }
} //end of namespace sstmac


