/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

// hdtorus.h: Interface for torus networks.
//
// Author: Ali Pinar <apinar@sandia.gov>

#ifndef SSTMAC_HARDWARE_NETWORK_TOPOLOGY_DRAGONFLY_H_INCLUDED
#define SSTMAC_HARDWARE_NETWORK_TOPOLOGY_DRAGONFLY_H_INCLUDED

#include <sstmac/hardware/network/switches/switchtopology.h>
#include <sstmac/hardware/network/topology/topology.h>

namespace sstmac
{
  namespace hw
  {

    class dragonfly : public switchtopology
    {

    protected:
      dragonfly(long x, long y, long g, int nps);

      int x_;
      int y_;
      int g_;
      int nps_;

    public:
      typedef boost::intrusive_ptr<dragonfly> ptr;

      static ptr
      construct(sim_parameters::ptr params);

      virtual std::string
      to_string() const
      {
        return "dragonfly";
      }

      void
      get_coords(long uid, int &x, int &y, int &g);

      long
      get_uid(int x, int y, int g);

      virtual int64_t
      n() const
      {
        return x_ * y_ * g_;
      }

      virtual
      ~dragonfly()
      {
      }

      virtual int64_t
      topology_size() const
      {
        return n();
      }

      virtual int
      radix() const
      {
        return x_ + y_ + 5;
      }

      std::string
      coord_string(int x, int y, int g);

      virtual void
      connect_switches(switchtopology::maptype &switches,
          sim_parameters::ptr params,
          std::string linkbwname = "network_bandwidth_link");

      virtual router::ptr
      create_router(nodeaddress::ptr addr, sim_parameters::ptr params);

      virtual int64_t
      diameter() const
      {
        return -1;
      }

    };

  }
} //end of namespace sstmac

#endif
