/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/hardware/nic/networkinterface.h>
#include <sstmac/software/process/operatingsystem.h>
#include <sstmac/hardware/network/congestion/interconnect.h>

namespace sstmac
{
  namespace hw
  {

    debug networkinterface::dbg_;
    stat_spyplot::ptr networkinterface::spy_;
    stat_spyplot::ptr networkinterface::spy_bytes_;
    stat_histogram<long>::ptr networkinterface::hist_msg_size_;
    stat_sum::ptr networkinterface::total_lat_;

    networkinterface::networkinterface()
    {
      if (!dbg_.is_real())
        {
          dbg_ = debug("<debug> nic");
          logger::register_logger(&dbg_);
        }

    }

    void
    networkinterface::finish()
    {
        if (lib_mem_){
            lib_mem_->get_os()->unregister_lib(lib_mem_);
            lib_mem_->unregister_all_libs();
            lib_mem_ = 0;
        }
        parent_ = 0;
        interconn_ = 0;
    }

    void
    networkinterface::init(const node::ptr &parent,
        const sstmac::hw::interconnect::ptr &interconn)
    {

      interconn_ = interconn;
      parent_ = parent;
      sw::lib_compute_memmove::ptr thelib = boost::dynamic_pointer_cast<
          sw::lib_compute_memmove>(
          parent->get_os()->get_lib(
              sw::lib_compute_memmove::get_name("nic-lib-mem")));

      if (!thelib)
        {
          lib_mem_ = sw::lib_compute_memmove::construct("nic-lib-mem");
          lib_mem_->init_callback();
          parent->get_os()->register_lib(lib_mem_);
        }
      else
        {
          lib_mem_ = thelib;
        }

      if (parent->get_id())
        {
          ptr s(this);
          interconn_->set_endpoint(parent->get_id(), s);
        }
    }

    long
    networkinterface::failure_id() const
    {
      return parent_->get_id()->unique_id();
    }

    sstmac::hw::topology::ptr
    networkinterface::get_topology()
    {
      if (interconn_)
        return interconn_->get_topology();
      else
        return sstmac::hw::topology::ptr();
    }

    networkinterface::~networkinterface()
    {
    }

    void
    networkinterface::validate_msg_supported(const sst_message::ptr &msg) const
    {
      return;
      /*switch (msg->get_type())
       {
       case sst_message::MPI:
       case sst_message::HPX:
       case sst_message::SEND_ACK:
       case sst_message::ACTOR:
       case sst_message::NETWORK:
       case sst_message::FILEMETA:
       case sst_message::FILEDATA:
       return; //valid
       }
       std::string errormsg = to_string()
       + ": network interface card received invalid message type";
       errormsg += sst_message::str(msg->get_type());
       throw unimplementederror(errormsg);*/
    }

  }
} // end of namespace sstmac.
