/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <math.h>

#include <sstmac/software/libraries/compute/lib_compute_loops.h>

#include <sstmac/software/process/key.h>

namespace sstmac
{
  namespace sw
  {

    int lib_compute_loops::mem_op_size_ = 16;

    double lib_compute_loops::mem_line_ratio_ = -1;
    double lib_compute_loops::flop_line_ratio_ = -1;
    bool lib_compute_loops::do_loops_ = true;

    void
    lib_compute_loops::compute_loop(int from, int to, int numlines, long tid)
    {
      if (do_loops_)
        {
          int loop = to - from + 1; //+1 for fortran loops

          doing_memory_ = true;
          lib_compute_memmove::read(
              loop * mem_line_ratio_ * numlines * mem_op_size_, tid);
          doing_memory_ = false;

          eventdata::ptr inst = eventdata::construct(get_lib_name(),
              timestamp(0));
          inst->set_event_value(eventdata::INST_FLOP, loop * flop_line_ratio_ * numlines);
          lib_compute_inst::compute_inst(inst);
        }
    }

    void
    lib_compute_loops::compute_loop2(int from1, int to1, int from2, int to2,
        int numlines, long tid)
    {
      if (do_loops_)
        {
          int loop1 = to1 - from1 + 1; //+1 for fortran loops
          int loop2 = to2 - from2 + 1;
          int loop = loop1 * loop2;

          doing_memory_ = true;
          lib_compute_memmove::read(
              loop * mem_line_ratio_ * numlines * mem_op_size_, tid);
          doing_memory_ = false;

          eventdata::ptr inst = eventdata::construct(get_lib_name(),
              timestamp(0));
          inst->set_event_value(eventdata::INST_FLOP, loop * flop_line_ratio_ * numlines);
          lib_compute_inst::compute_inst(inst);
        }
    }

    void
    lib_compute_loops::compute_loop3(int from1, int to1, int from2, int to2,
        int from3, int to3, int numlines, long tid)
    {
      if (do_loops_)
        {
          int loop1 = to1 - from1 + 1; //+1 for fortran loops
          int loop2 = to2 - from2 + 1;
          int loop3 = to3 - from3 + 1;
          int loop = loop1 * loop2 * loop3;

          doing_memory_ = true;
          lib_compute_memmove::read(
              loop * mem_line_ratio_ * numlines * mem_op_size_, tid);
          doing_memory_ = false;

          eventdata::ptr inst = eventdata::construct(get_lib_name(),
              timestamp(0));
          inst->set_event_value(eventdata::INST_FLOP, loop * flop_line_ratio_ * numlines);
          lib_compute_inst::compute_inst(inst);
        }
    }

    void
    lib_compute_loops::compute_loop4(int from1, int to1, int from2, int to2,
        int from3, int to3, int from4, int to4, int numlines, long tid)
    {
      if (do_loops_)
        {
          int loop1 = to1 - from1 + 1; //+1 for fortran loops
          int loop2 = to2 - from2 + 1;
          int loop3 = to3 - from3 + 1;
          int loop4 = to4 - from4 + 1;
          int loop = loop1 * loop2 * loop3 * loop4;

          doing_memory_ = true;
          lib_compute_memmove::read(
              loop * mem_line_ratio_ * numlines * mem_op_size_, tid);
          doing_memory_ = false;

          eventdata::ptr inst = eventdata::construct(get_lib_name(),
              timestamp(0));
          inst->set_event_value(eventdata::INST_FLOP, loop * flop_line_ratio_ * numlines);
          lib_compute_inst::compute_inst(inst);
        }
    }

    void
    lib_compute_loops::compute_fft()
    {
      if (do_loops_)
        {
          eventdata::ptr inst = eventdata::construct(get_lib_name(),
              timestamp(0));
          inst->set_event_value(eventdata::INST_FLOP, 500);
          lib_compute_inst::compute_inst(inst);
        }
    }

  }
} //end of namespace sstmac
