/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_COMPUTE_MEM_ACCESS_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_COMPUTE_MEM_ACCESS_H_INCLUDED

#include <sstmac/common/messages/sst_message.h>
#include <sstmac/common/messages/eventdata.h>
#include <sstmac/common/eventcallback.h>
#include <sstmac/software/ami/ami.h>

namespace sstmac
{
  namespace sw
  {

    class mem_access : public eventdata
    {
    protected:
      mem_access(long bytes,
                 const std::string& libn,
                 AMI::AMI_COMP_FUNC access_type,
                 long srcth, long destth) :
          eventdata(libn),
          func_(access_type),
          bytes_(bytes),
          srcthread_(srcth),
          dstthread_(destth)
      {
      }

      long bytes_;

      AMI::AMI_COMP_FUNC func_;

    public:
      typedef boost::intrusive_ptr<mem_access> ptr;

      static ptr
      construct(long bytes,
                const std::string& libn,
                AMI::AMI_COMP_FUNC access_type,
                long thr)
      {
        return ptr(new mem_access(bytes, libn, access_type, thr, thr));
      }

      static ptr
      construct(long bytes,
                const std::string& libn,
                AMI::AMI_COMP_FUNC access_type,
                long srcth, long dstth)
      {
        return ptr(new mem_access(bytes, libn, access_type, srcth, dstth));
      }


      AMI::AMI_COMP_FUNC
      get_function() const
      {
          return func_;
      }

      virtual long
      get_byte_length() const
      {
        return bytes_;
      }

      long
      source_thread(const sw::operatingsystem::ptr& os) const
      {
          return srcthread_;
      }

      long
      dest_thread(const sw::operatingsystem::ptr& os) const
      {
          return dstthread_;
      }

      long uid_;
      long srcthread_;
      long dstthread_;
    };

  }
}

#endif
