/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/software/libraries/mpi/mpi_implementation/basicmpi.h>
#include <sstmac/software/libraries/mpi/mpi_protocol/mpiprotocol.h>
#include <sstmac/common/sim_parameters.h>

namespace sstmac {
    namespace sw {

basic_mpi_implementation::basic_mpi_implementation() :
    mpi_implementation(24), //envelope size
    handsize_(0)
{
    handsize_ = sstmac_env::params->get_optional_int_param("mpi_handshake_size", 1 << 16);
}

mpiprotocol::ptr
basic_mpi_implementation::get_protocol(long bytes, bool intranode, bool rsend) const
{
    if (intranode){
        if (rsend || bytes > handsize_)
        {
            return mpiprotocol::rendezvous_mmap_protocol;
        }
        else
        {
            return mpiprotocol::eager0_mmap_protocol;
        }
    }
    else {
        if (rsend || bytes <= handsize_)
        {
            return mpiprotocol::eager0_socket_protocol;
        }
        else
        {
            return mpiprotocol::rendezvous_socket_protocol;
        }
    }
}

mpi_implementation::ptr
basic_mpi_implementation::construct()
{
    return new basic_mpi_implementation();
}

    }
}
