/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/software/libraries/mpi/mpi_implementation/basicmpi.h>
#include <sstmac/software/libraries/mpi/mpi_implementation/rdmampi.h>
#include <sstmac/software/libraries/mpi/mpi_implementation/simplempi.h>
#include <sstmac/software/libraries/mpi/mpi_protocol/mpiprotocol.h>
#include <sstmac/common/sim_parameters.h>

namespace sstmac
{
  namespace sw
  {

    mpi_implementation::mpi_implementation(int64_t envelope) :
      envelope_(envelope)
    {
    }

    mpi_implementation::ptr
    mpi_implementation::construct(const std::string &name)
    {
      if (name == "basic")
        {
          return basic_mpi_implementation::construct();
        }
      else if (name == "rdma" || name == "rdmampi" || name == "rdma_mpi")
        {
          return rdmampi::construct();
        }
      else if (name == "simple")
        {
          return simple_mpi_implementation::construct();
        }
      else
        {
          sst_throw(valueerror, "mpi_implementation: invalid implementation type %s", name.c_str());
        }
    }

    int64_t
    mpi_implementation::get_envelope_size() const
    {
      return envelope_;
    }

  }
}

