/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/software/libraries/mpi/mpi_implementation/rdmampi.h>
#include <sstmac/software/libraries/mpi/mpi_protocol/mpiprotocol.h>
#include <sstmac/common/sim_parameters.h>

namespace sstmac {
    namespace sw {

        rdmampi::rdmampi()
            : mpi_implementation(24) //envelope size
        {
            smp_single_copy_size_ = sstmac_env::params->get_optional_int_param("smp_single_copy_size", 8192);
            max_eager_msg_size_ = sstmac_env::params->get_optional_int_param("max_eager_msg_size", 8192);
            max_vshort_msg_size_ = sstmac_env::params->get_optional_int_param("max_vshort_msg_size", 1024);
        }

        mpiprotocol::ptr
        rdmampi::get_protocol(long bytes, bool intranode, bool rsend) const
        {
            if (intranode){
                if (rsend || bytes > smp_single_copy_size_)
                    return mpiprotocol::rendezvous_mmap_protocol;
                else
                    return mpiprotocol::eager0_mmap_protocol;
            }
            else
            {
                if (rsend || bytes <= max_vshort_msg_size_)
                    return mpiprotocol::eager0_rdma_protocol;
                else if (bytes <= max_eager_msg_size_)
                    return mpiprotocol::eager1_rdma_singlecpy_protocol;
                else
                    return mpiprotocol::rendezvous_rdma_protocol;
            }
        }

        rdmampi::ptr
        rdmampi::construct()
        {
            return new rdmampi;
        }

        int64_t
        rdmampi::max_eager_msg_size() const
        {
            return max_eager_msg_size_;
        }

        int64_t
        rdmampi::max_vshort_msg_size() const
        {
            return max_vshort_msg_size_;
        }

    }
}
