#include <sstmac/software/libraries/mpi/mpi_protocol/mpiprotocol.h>
#include <sstmac/software/libraries/mpi/mpiserver.h>

namespace sstmac {
    namespace sw {


void
rendezvous_mmap::send_header(const mpiqueue::ptr &queue, const mpimessage::ptr &msg)
{
    SSTMAC_DEBUG << "mpiqueue[" << queue->id_string()
          << "]: start rendezvous mmap send\n";
    queue->server()->send(msg);
}

void
rendezvous_mmap::handle_header(const mpiqueue::ptr &queue, const mpimessage::ptr &msg)
{
    mpiqueue::recvrequest::ptr req = queue->find_pending_request(msg);
    if (req)
      req->handle(msg);

    queue->notify_probes(msg);

    SSTMAC_DEBUG << "mpiqueue[" << queue->id_string()
          << "]:: handle rendezvous mmap header done \n";
}

void
rendezvous_mmap::handle_mpi_ack(
    const mpiqueue::ptr& queue,
    const mpimessage::ptr& msg,
    const mpiqueue::sendrequest::ptr& req
)
{
    msg->ack_to_payload();
    queue->server()->send(msg);
}


void
rendezvous_mmap::handle_payload(const mpiqueue::ptr &queue, const mpimessage::ptr &msg)
{
    SSTMAC_DEBUG << "mpiqueue[" << queue->id_string()
          << "]::incoming rendezvous payload "
          << msg->to_string() << "\n";
    mpiqueue::ack_sent_t::iterator it = queue->mpi_ack_sent_.find(msg->id());
    if (it == queue->mpi_ack_sent_.end())
    {
        throw illformederror("mpiqueue::incoming_message:  Data message "
          "without a matching ack.");
    }
    mpiqueue::recvrequest::ptr recver = it->second;
    queue->mpi_ack_sent_.erase(it);
    queue->server()->buffered_recv(msg, recver);
}

    }
}
