#include <sstmac/software/libraries/mpi/mpi_protocol/mpiprotocol.h>
#include <sstmac/software/libraries/mpi/mpiserver.h>

namespace sstmac {
    namespace sw {


void
rendezvous_rdma::send_header(const mpiqueue::ptr &queue, const mpimessage::ptr &msg)
{
    SSTMAC_DEBUG << "mpiqueue[" << queue->id_string()
          << "]: start rendezvous rdma send\n";
    msg->set_needs_ack(false);  //we don't care about the nic ack for this
    queue->server()->send(msg);
}

void
rendezvous_rdma::handle_header(const mpiqueue::ptr &queue, const mpimessage::ptr &msg)
{
    mpiqueue::recvrequest::ptr req = queue->find_pending_request(msg);
    if (req)
      req->handle(msg);

    queue->notify_probes(msg);

    SSTMAC_DEBUG << "mpiqueue[" << queue->id_string()
          << "]:: handle rendezvous rdma header done \n";
}

void
rendezvous_rdma::handle_mpi_ack(
    const mpiqueue::ptr& queue,
    const mpimessage::ptr &msg,
    const mpiqueue::sendrequest::ptr& req
)
{
}

void
rendezvous_rdma::handle_payload(const mpiqueue::ptr &queue, const mpimessage::ptr &msg)
{
    SSTMAC_DEBUG << "mpiqueue[" << queue->id_string()
          << "]::incoming_message:  " << "Completion of a message.\n";
    mpiqueue::ack_sent_t::iterator it = queue->rdma_ack_waiting_.find(msg->id());
    if (it == queue->rdma_ack_waiting_.end())
    {
        throw illformederror("mpiqueue::incoming_message:  rdma message "
          "without a notification of waiting ack.");
    }
    mpiqueue::recvrequest::ptr recver = it->second;
    queue->rdma_ack_waiting_.erase(it);
    recver->handle(msg);
}

void
rendezvous_rdma::recv_header(
    const mpiqueue::ptr &queue,
    const mpimessage::ptr &msg,
    const mpiqueue::recvrequest::ptr& req
)
{
    msg->mutate(sst_message::GNI_HEADER_TO_RDMA_GET_REQ);
    msg->set_needs_ack(true); //this has to ack the RDMA get back to MPI
    queue->rdma_ack_waiting_[msg->id()] = req;
    queue->server()->send(msg);
}

void
rendezvous_rdma::recv_payload(
    const mpiqueue::ptr &queue,
    const mpimessage::ptr &msg,
    const mpiqueue::recvrequest::ptr& req
)
{
}

    }
}
