#include <sstmac/software/libraries/mpi/mpi_protocol/mpiprotocol.h>
#include <sstmac/software/libraries/mpi/mpiserver.h>

namespace sstmac {
    namespace sw {


void
rendezvous_protocol::recv_header(
    const mpiqueue::ptr &queue,
    const mpimessage::ptr &msg,
    const mpiqueue::recvrequest::ptr& req
)
{
    //ack here
    queue->send_mpi_ack(msg, req);
}

void
rendezvous_protocol::recv_payload(
    const mpiqueue::ptr &queue,
    const mpimessage::ptr &msg,
    const mpiqueue::recvrequest::ptr& req
)
{
    SSTMAC_DEBUG << "mpiqueue[" << queue->id_string()
          << "]: recv rendezvous payload "
          << msg->to_string() << "\n";
}

bool
rendezvous_protocol::handshake_only() const
{
    return true;
}

bool
rendezvous_protocol::need_mpi_ack() const
{
    return true;
}

void
rendezvous_socket::send_header(const mpiqueue::ptr &queue, const mpimessage::ptr &msg)
{
    SSTMAC_DEBUG << "mpiqueue[" << queue->id_string()
          << "]: start rendezvous socket send\n";
    queue->server()->send(msg);
}

void
rendezvous_socket::handle_header(const mpiqueue::ptr &queue, const mpimessage::ptr &msg)
{
    mpiqueue::recvrequest::ptr req = queue->find_pending_request(msg);
    if (req)
      req->handle(msg);

    queue->notify_probes(msg);

    SSTMAC_DEBUG << "mpiqueue[" << queue->id_string()
          << "]:: handle new rendezvous socket header done \n";
}

void
rendezvous_socket::handle_mpi_ack(
    const mpiqueue::ptr& queue,
    const mpimessage::ptr& msg,
    const mpiqueue::sendrequest::ptr& req
)
{
    msg->ack_to_payload();
    queue->server()->send(msg);
}

void
rendezvous_socket::handle_payload(const mpiqueue::ptr &queue, const mpimessage::ptr &msg)
{
    SSTMAC_DEBUG << "mpiqueue[" << queue->id_string()
          << "]::incoming rendezvous payload "
          << msg->to_string() << "\n";
    mpiqueue::ack_sent_t::iterator it = queue->mpi_ack_sent_.find(msg->id());
    if (it == queue->mpi_ack_sent_.end())
    {
        throw illformederror("mpiqueue::incoming_message:  Data message "
          "without a matching ack.");
    }
    mpiqueue::recvrequest::ptr recver = it->second;
    queue->mpi_ack_sent_.erase(it);
    recver->handle(msg);
}

    }
}

