/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/software/libraries/mpi/mpi_queue/mpiqueue-recvrequest.h>
#include <sstmac/software/libraries/mpi/mpi_protocol/mpiprotocol.h>
#include <sstmac/software/libraries/mpi/mpimessage.h>
#include <sstmac/common/debug.h>
#include <sstmac/common/errors.h>

namespace sstmac { namespace sw
{

  // Pointer types.
  
  

  //
  // Hello.
  //
  mpiqueue::recvrequest::recvrequest(const mpirequest_t key,
      const mpiqueue::ptr& queue, int count, const mpitype &type,
      const mpiid &source, const mpitag &tag, const mpicomm::ptr &comm,
      mpimessage::category cat, eventhandler::ptr &completion) :
    queue_(queue), source_(source), tag_(tag), comm_(comm),
        cat_(cat), count_(count), type_(type), key_(key),
        completion_(completion)
  {
  }



  //
  // Hello.
  //
 mpiqueue::recvrequest::ptr
  mpiqueue::recvrequest::construct(const mpirequest_t key,
      const mpiqueue::ptr& queue, int count, const mpitype &type,
      const mpiid &source, const mpitag &tag, const mpicomm::ptr &comm,
      mpimessage::category cat, eventhandler::ptr &completion)
  {
    recvrequest::ptr retval(
        new recvrequest(key, queue, count, type, source, tag, comm, cat,
            completion));

    return retval;
  }

  //
  // Goodbye.
  //
  mpiqueue::recvrequest::~recvrequest()
  {
   // std::cout << "calling recvrequest destructor" << std::endl;
  }

  //
  // We be done.
  //
  void
  mpiqueue::recvrequest::handle(const mpimessage::ptr& mess)
  {

      if (mess->count() < 0){
          SSTMAC_DEBUG << "mpiqueue[" << queue_->id_string() << "]:acking "
                       << mess->to_string() << "\n";
          mess->get_protocol()->recv_header(queue_, mess, recvrequest::ptr(this));
      }
      else {
          mess->get_protocol()->recv_payload(queue_, mess, recvrequest::ptr(this));

          SSTMAC_DEBUG << "mpiqueue[" << queue_->id_string() << "]:completing "
                       << mess->to_string() << "\n";
          key_->complete(mess);
          completion_->handle(mess);
      }

  }

} } // end of namespace sstmac
