/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/software/process/app.h>
#include <sstmac/software/process/operatingsystem.h>
#include <sstmac/software/services/launch/launchinfo.h>
namespace sstmac
{
  namespace sw
  {

    debug app::dbg_;

    app::app(softwareid id) :
      id_(id)
    {
      if (!dbg_.is_real())
        {
          dbg_ = debug("<debug> thread | app", logger::green);
          logger::register_logger(&dbg_);
        }
    }

    app::~app()
    {
    }

    void
    app::set_launch_info(boost::intrusive_ptr<const launchinfo> &linf)
    {
      linfo_ = linf;
    }

    void
    app::kill()
    {
        linfo_ = 0;
        thread::kill();
    }

    boost::intrusive_ptr<const launchinfo>
    app::get_launch_info()
    {
      return linfo_;
    }

    void
    app::compute(timestamp time)
    {

      if (!computetime_)
        {
          computetime_ = lib_compute_time::construct(id_);
          register_lib(computetime_);
        }
      computetime_->compute(time);
    }

    void
    app::compute_inst(const eventdata::ptr &cmsg)
    {
      if (!computeinst_)
        {
          computeinst_ = lib_compute_inst::construct(id_);
          register_lib(computeinst_);
        }
      computeinst_->compute_inst(cmsg);
    }

    void
    app::compute_block_read(long bytes)
    {
      if (!computememmove_)
        {
          computememmove_ = lib_compute_memmove::construct(id_);
          register_lib(computememmove_);
        }
      computememmove_->read(bytes);
    }

    void
    app::compute_block_write(long bytes)
    {
      if (!computememmove_)
        {
          computememmove_ = lib_compute_memmove::construct(id_);
          register_lib(computememmove_);
        }
      computememmove_->write(bytes);
    }

    void
    app::compute_block_memcpy(long bytes)
    {
      if (!computememmove_)
        {
          computememmove_ = lib_compute_memmove::construct(id_);
          register_lib(computememmove_);
        }
      computememmove_->copy(bytes);
    }

    void
    app::sleep(timestamp time)
    {
      if (!sleeplib_)
        {
          sleeplib_ = lib_sleep::construct(id_);
          register_lib(sleeplib_);
        }
      sleeplib_->sleep(time);
    }

    std::string
    app::get_compute_name()
    {

      if (!computetime_)
        {
          computetime_ = lib_compute_time::construct(id_);
          register_lib(computetime_);
        }
      return computetime_->get_lib_name();
    }

    std::string
    app::get_compute_inst_name()
    {
      if (!computeinst_)
        {
          computeinst_ = lib_compute_inst::construct(id_);
          register_lib(computeinst_);
        }
      return computeinst_->get_lib_name();
    }

    std::string
    app::get_compute_memmove_name()
    {
      if (!computememmove_)
        {
          computememmove_ = lib_compute_memmove::construct(id_);
          register_lib(computememmove_);
        }
      return computememmove_->get_lib_name();
    }

    std::string
    app::get_sleep_name()
    {
      if (!sleeplib_)
        {
          sleeplib_ = lib_sleep::construct(id_);
          register_lib(sleeplib_);
        }
      return sleeplib_->get_lib_name();
    }

  }
}
