#ifndef SSTMAC_SOFTWARE_PROCESS_GRAPHVIZ_H
#define SSTMAC_SOFTWARE_PROCESS_GRAPHVIZ_H

#include <map>
#include <sstmac/common/ptr_type.h>
#include <sstmac/common/stats/stat_logger.h>

namespace sstmac {
    namespace sw {

    class GraphViz :
        public stat_logger,
        public ptr_type
    {
        public:
            typedef boost::intrusive_ptr<GraphViz> ptr;

            std::string to_string() const
            {
                return "grahpviz";
            }

            static ptr
            construct();

            void simulation_finished(timestamp end);

            void trace(long count);

        private:

            GraphViz();

            typedef std::pair<long, long long> graphviz_call;
            class Trace :
                public ptr_type
            {

                private:

                    std::map<std::string, graphviz_call> calls_;

                    long long self_;

                    std::string fxn_;

                    GraphViz::ptr parent_;

                    Trace(const GraphViz::ptr& parent, const std::string& fxn);

                public:
                    typedef boost::intrusive_ptr<Trace> ptr;

                    static ptr construct(const GraphViz::ptr& parent, const std::string& fxn);

                    std::string to_string() const
                    {
                        return "graphviz trace";
                    }

                    std::string
                    summary() const;

                    std::string fxn() const;

                    void add_call(const std::string& fxn, int ncalls, long count);

                    void add_self(long count);

            };

            void add_call(int ncalls, long count, void* fxn, void* callfxn);

            void get_info(void* addr, std::string& fxn);

            std::map<std::string, Trace::ptr> traces_;

            std::map<void*, std::string> fxns_;

            friend class Trace;

            Trace::ptr get_trace(const std::string& fxn);


    };

    }
}


#endif // GRAPHVIZ_H
