/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/software/services/launch/launchinfo.h>
#include <sstmac/software/process/app.h>

namespace sstmac
{
  namespace sw
  {

    launchinfo::launchinfo(const app::ptr& app, appid a, const sim_parameters::ptr &p,
        timestamp start, long num_tasks, long ntask_per_node, long extra,
        const std::vector<int>& core_affinities) :
      apptype_(app), aid_(a), params_(p), start_(start), num_tasks_(num_tasks),
          ntask_per_node_(ntask_per_node), extra_nodes_(extra),
          core_affinities_(core_affinities)
    {
    }

    launchinfo::ptr
    launchinfo::construct(std::string t, appid a, sim_parameters::ptr &p,
        timestamp start, long num_tasks, long ntask_per_node, long extra,
        const std::vector<int>& core_affinities)
    {

      app::ptr theapp = SSTAppFactory::get_app(t, p);

      return ptr(
          new launchinfo(theapp, a, p, start, num_tasks, ntask_per_node, extra,
              core_affinities));
    }

    launchinfo::ptr
    launchinfo::construct(const app::ptr& app, appid a, const sim_parameters::ptr &p,
        timestamp start, long num_tasks, long ntask_per_node, long extra,
        const std::vector<int>& core_affinities)
    {
      return ptr(
          new launchinfo(app, a, p, start, num_tasks, ntask_per_node, extra,
              core_affinities));
    }

    int
    launchinfo::core_affinity(int intranode_rank) const
    {
      if (core_affinities_.size() > 0) //we are assigned
        {
          return core_affinities_[intranode_rank];
        }
      else
        {
          return thread::no_core_affinity;
        }
    }

    launchinfo::~launchinfo()
    {
    }

  }
}
