/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_BACKENDS_NATIVE_LAUNCH_launchinfo_H_INCLUDED
#define SSTMAC_BACKENDS_NATIVE_LAUNCH_launchinfo_H_INCLUDED

#include <sstmac/common/ptr_type.h>
#include <sstmac/common/timestamp.h>
#include <list>
#include <sstmac/software/process/appid.h>

#include <sstmac/common/factories/app_factory.h>

#include <sstmac/common/sim_parameters.h>

namespace sstmac
{
  namespace sw
  {

    class app;

    class launchinfo : public ptr_type
    {
    public:
      typedef boost::intrusive_ptr<const launchinfo> ptr;

      timestamp start_;

      boost::intrusive_ptr<app> apptype_;

      appid aid_;

      long num_tasks_;

      long ntask_per_node_;

      long extra_nodes_;

      std::vector<int> core_affinities_;

      const sim_parameters::ptr params_;

    protected:
      launchinfo(const boost::intrusive_ptr<app>& app,
          appid a, const sim_parameters::ptr &p,
          timestamp start, long num_tasks, long ntask_per_node, long extra,
          const std::vector<int>& core_affinities);

    public:
      static ptr
      construct(std::string t, appid a, sim_parameters::ptr &p,
          timestamp start, long num_tasks, long ntask_per_node, long extra,
          const std::vector<int>& core_affinities);

      static ptr
      construct(
          const boost::intrusive_ptr<app>& app,
          appid a, const sim_parameters::ptr &p,
          timestamp start, long num_tasks, long ntask_per_node, long extra,
          const std::vector<int>& core_affinities);

      virtual std::string
      to_string() const
      {
        return "launchinfo";
      }

      int core_affinity(int intranode_rank) const;

      virtual ~launchinfo();

    };

  }
}
#endif
