/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include "fake_app.h"

#include <sstmac/software/libraries/mpi/sstmac_mpi.h>
#include <sstmac/sstmacro.h>
#include <sstmac/common/factories/app_factory.h>

namespace fail_launch
{

  using namespace std;
  using namespace sstmac;
  using namespace sstmac::sw;
  SSTRegisterApp("fakeapp",  fakeapp);

  void
  fakeapp::consume_params(const sim_parameters::ptr &params)
  {
    if (params->has_param("fakeapp_time"))
      {
        time_ = params->get_time_param("fakeapp_time");
      }
    else
      {
        std::stringstream ss;
        ss << "fakeapp" << id_.app_.id << "_time";
        time_ = params->get_time_param(ss.str());
      }

  }

  //
  // Go.
  //
  void
  fakeapp::skeleton_main()
  {

    MPI_Init(NULL, NULL);
    int rank;
    MPI_Comm_rank(MPI_Comm_World(), &rank);

    SSTMAC_DEBUG << "fakeapp" << id_.app_.id << "[" << rank << "]: starting up \n";

    sleep(time_);

    SSTMAC_DEBUG << "fakeapp" << id_.app_.id << "[" << rank << "]: all done \n";
    MPI_Finalize();
  }

} // end of namespace sstmac.
